@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -26,18 +26,16 @@
 
 #include <stddef.h>
 
-#if defined(__cplusplus)
-#include <new>
-#endif
-
-#define QT_VERSION_STR   "4.0.1"
+#define QT_VERSION_STR   "4.1.1"
 /*
    QT_VERSION is (major << 16) + (minor << 8) + patch.
- */
-#define QT_VERSION 0x040001
+*/
+#define QT_VERSION 0x040101
+
+#define QT_PACKAGEDATE_STR "2006-02-16"
 
 #if !defined(QT_BUILD_MOC)
-#include "QtCore/qconfig.h"
+#include <QtCore/qconfig.h>
 #endif
 
 /*
@@ -225,6 +223,11 @@
 #  if defined(__INTEL_COMPILER)
 #    define Q_CC_INTEL
 #  endif
+/* x64 does not support mmx intrinsics on windows */
+#  if (defined(Q_OS_WIN64) && defined(_M_X64))
+#    undef QT_HAVE_SSE
+#  endif
+
 
 #elif defined(__BORLANDC__) || defined(__TURBOC__)
 #  define Q_CC_BOR
@@ -233,7 +236,7 @@
 #    define Q_NO_BOOL_TYPE
 #    define Q_NO_EXPLICIT_KEYWORD
 #  endif
-#  define Q_NO_USING_KEYWORD /* ### check "using" status */
+#  define Q_NO_USING_KEYWORD
 
 #elif defined(__WATCOMC__)
 #  define Q_CC_WAT
@@ -330,7 +333,7 @@
    Compaq C++ V6.3-002.
    This compiler is different enough from other EDG compilers to handle
    it separately anyway. */
-#elif defined(__DECCXX)
+#elif defined(__DECCXX) || defined(__DECC)
 #  define Q_CC_DEC
 /* Compaq C++ V6 compilers are EDG-based but I'm not sure about older
    DEC C++ V5 compilers. */
@@ -355,9 +358,18 @@
 /* avoid undefined symbol problems with out-of-line template members */
 #  define Q_OUTOFLINE_TEMPLATE inline
 
+/* The Portland Group C++ compiler is based on EDG and does define __EDG__
+   but the C compiler does not */
+#elif defined(__PGI)
+#  define Q_CC_PGI
+#  if defined(__EDG__)
+#    define Q_CC_EDG
+#  endif
+
 /* Compilers with EDG front end are similar. To detect them we test:
    __EDG documented by SGI, observed on MIPSpro 7.3.1.1 and KAI C++ 4.0b
-   __EDG__ documented in EDG online docs, observed on Compaq C++ V6.3-002 */
+   __EDG__ documented in EDG online docs, observed on Compaq C++ V6.3-002
+   and PGI C++ 5.2-4 */
 #elif defined(__EDG) || defined(__EDG__)
 #  define Q_CC_EDG
 /* From the EDG documentation (does not seem to apply to Compaq C++):
@@ -388,10 +400,6 @@
 #  elif defined(__INTEL_COMPILER)
 #    define Q_CC_INTEL
 
-/* The Portland Group compiler is based on EDG and does define __EDG__ */
-#  elif defined(__PGI)
-#    define Q_CC_PGI
-
 /* Never tested! */
 #  elif defined(__ghs)
 #    define Q_CC_GHS
@@ -485,9 +493,19 @@
 
 #ifndef Q_CONSTRUCTOR_FUNCTION
 # define Q_CONSTRUCTOR_FUNCTION(AFUNC) \
-   static const int AFUNC_init_var = AFUNC();
+   static const int AFUNC ## __init_variable__ = AFUNC();
 #endif
 
+#ifndef Q_DESTRUCTOR_FUNCTION
+# define Q_DESTRUCTOR_FUNCTION(AFUNC) \
+    class AFUNC ## __dest_class__ { \
+    public: \
+       inline AFUNC ## __dest_class__() { } \
+       inline ~ AFUNC ## __dest_class__() { AFUNC(); } \
+    } AFUNC ## __dest_instance__;
+#endif
+
+
 /*
    The window system, must be one of: (Q_WS_x)
 
@@ -528,7 +546,7 @@
 
 
 /*
-  Size-dependent types (architechture-dependent byte order)
+   Size-dependent types (architechture-dependent byte order)
 */
 
 typedef signed char qint8;         /* 8 bit signed */
@@ -561,6 +579,9 @@ typedef quint64 qulonglong;
 #define Q_INIT_RESOURCE(name) \
     do { extern int qInitResources_ ## name (); \
     qInitResources_ ## name (); } while (0)
+#define Q_CLEANUP_RESOURCE(name) \
+    do { extern int qCleanupResources_ ## name (); \
+        qCleanupResources_ ## name (); } while (0)
 
 #if defined(__cplusplus)
 
@@ -636,7 +657,7 @@ typedef unsigned long ulong;
 #  endif
 #endif
 #if defined(QT3_SUPPORT_WARNINGS)
-#  if !defined(QT_COMPAT_WARNINGS) //also enable compat
+#  if !defined(QT_COMPAT_WARNINGS) /* also enable compat */
 #    define QT_COMPAT_WARNINGS
 #  endif
 #  undef QT3_SUPPORT
@@ -645,8 +666,8 @@ typedef unsigned long ulong;
 #  define QT3_SUPPORT_VARIABLE Q_DECL_VARIABLE_DEPRECATED
 #  undef QT3_SUPPORT_CONSTRUCTOR
 #  define QT3_SUPPORT_CONSTRUCTOR explicit Q_DECL_CONSTRUCTOR_DEPRECATED
-#elif defined(QT3_SUPPORT) //define back to nothing
-#  if !defined(QT_COMPAT) //also enable qt3 support
+#elif defined(QT3_SUPPORT) /* define back to nothing */
+#  if !defined(QT_COMPAT) /* also enable qt3 support */
 #    define QT_COMPAT
 #  endif
 #  undef QT3_SUPPORT
@@ -663,7 +684,7 @@ typedef unsigned long ulong;
 #  define QT_COMPAT_VARIABLE Q_DECL_VARIABLE_DEPRECATED
 #  undef QT_COMPAT_CONSTRUCTOR
 #  define QT_COMPAT_CONSTRUCTOR explicit Q_DECL_CONSTRUCTOR_DEPRECATED
-#elif defined(QT_COMPAT) //define back to nothing
+#elif defined(QT_COMPAT) /* define back to nothing */
 #  undef QT_COMPAT
 #  define QT_COMPAT
 #  undef QT_COMPAT_VARIABLE
@@ -671,7 +692,7 @@ typedef unsigned long ulong;
 #  undef QT_COMPAT_CONSTRUCTOR
 #  define QT_COMPAT_CONSTRUCTOR
 #endif
-// moc compats (signals/slots)
+/* moc compats (signals/slots) */
 #ifndef QT_MOC_COMPAT
 #  if defined(QT3_SUPPORT)
 #    define QT_MOC_COMPAT QT3_SUPPORT
@@ -697,17 +718,25 @@ typedef unsigned long ulong;
 
 typedef int QNoImplicitBoolCast;
 
-//
-// Utility macros and inline functions
-//
+#if defined(QT_COORD_TYPE)
+typedef QT_COORD_TYPE qreal;
+#elif defined(__arm__)
+typedef float qreal;
+#else
+typedef double qreal;
+#endif
+
+/*
+   Utility macros and inline functions
+*/
 
 template <typename T>
 inline T qAbs(const T &t) { return t >= 0 ? t : -t; }
 
-inline int qRound(double d)
+inline int qRound(qreal d)
 { return d >= 0.0 ? int(d + 0.5) : int(d - int(d-1) + 0.5) + int(d-1); }
 
-inline qint64 qRound64(double d)
+inline qint64 qRound64(qreal d)
 { return d >= 0.0 ? qint64(d + 0.5) : qint64(d - qint64(d-1) + 0.5) + qint64(d-1); }
 
 template <typename T>
@@ -743,9 +772,9 @@ typedef unsigned long Q_ULONG;      /* word up to 64 bit unsigned */
 #  define QMIN(a, b) qMin((a), (b))
 #endif
 
-//
-// Data stream functions are provided by many classes (defined in qdatastream.h)
-//
+/*
+   Data stream functions are provided by many classes (defined in qdatastream.h)
+*/
 
 class QDataStream;
 
@@ -761,11 +790,6 @@ class QDataStream;
 #  if !defined(MAC_OS_X_VERSION_10_4)
 #       define MAC_OS_X_VERSION_10_4 MAC_OS_X_VERSION_10_3 + 1
 #  endif
-#if 0
-#  if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
-#    warning "Support for this version of Mac OS X is still preliminary."
-#  endif
-#endif
 #  if (MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_4)
 #    error "This version of Mac OS X is unsupported"
 #  endif
@@ -785,7 +809,7 @@ class QDataStream;
 #  define QT_NO_COP
 #endif
 
-# include "QtCore/qfeatures.h"
+# include <QtCore/qfeatures.h>
 
 #ifndef Q_DECL_EXPORT
 #  ifdef Q_OS_WIN
@@ -805,10 +829,11 @@ class QDataStream;
 #  endif
 #endif
 
-//
-// Create Qt DLL if QT_DLL is defined (Windows only)
-// or QT_SHARED is defined (Kylix only)
-//
+/*
+   Create Qt DLL if QT_DLL is defined (Windows only)
+   or QT_SHARED is defined (Kylix only)
+*/
+
 #if defined(Q_OS_WIN)
 #  if defined(QT_NODLL)
 #    undef QT_MAKEDLL
@@ -837,6 +862,11 @@ class QDataStream;
 #    else
 #      define Q_NETWORK_EXPORT Q_DECL_IMPORT
 #    endif
+#    if defined(QT_BUILD_SVG_LIB)
+#      define Q_SVG_EXPORT Q_DECL_EXPORT
+#    else
+#      define Q_SVG_EXPORT Q_DECL_IMPORT
+#    endif
 #    if defined(QT_BUILD_OPENGL_LIB)
 #      define Q_OPENGL_EXPORT Q_DECL_EXPORT
 #    else
@@ -863,6 +893,7 @@ class QDataStream;
 #    define Q_GUI_EXPORT Q_DECL_IMPORT
 #    define Q_SQL_EXPORT Q_DECL_IMPORT
 #    define Q_NETWORK_EXPORT Q_DECL_IMPORT
+#    define Q_SVG_EXPORT Q_DECL_IMPORT
 #    define Q_CANVAS_EXPORT Q_DECL_IMPORT
 #    define Q_OPENGL_EXPORT Q_DECL_IMPORT
 #    define Q_XML_EXPORT Q_DECL_IMPORT
@@ -885,6 +916,7 @@ class QDataStream;
 #    define Q_GUI_EXPORT Q_DECL_EXPORT
 #    define Q_SQL_EXPORT Q_DECL_EXPORT
 #    define Q_NETWORK_EXPORT Q_DECL_EXPORT
+#    define Q_SVG_EXPORT Q_DECL_EXPORT
 #    define Q_OPENGL_EXPORT Q_DECL_EXPORT
 #    define Q_XML_EXPORT Q_DECL_EXPORT
 #    define Q_COMPAT_EXPORT Q_DECL_EXPORT
@@ -893,15 +925,29 @@ class QDataStream;
 #    define Q_GUI_EXPORT
 #    define Q_SQL_EXPORT
 #    define Q_NETWORK_EXPORT
+#    define Q_SVG_EXPORT
 #    define Q_OPENGL_EXPORT
 #    define Q_XML_EXPORT
 #    define Q_COMPAT_EXPORT
 #  endif
 #endif
 
-//
-// System information
-//
+/*
+   No, this is not an evil backdoor. QT_BUILD_INTERNAL just exports more symbols
+   for Trolltech's internal unit tests. If you want slower loading times and more
+   symbols that can vanish from version to version, feel free to define QT_BUILD_INTERNAL.
+*/
+#if defined(QT_BUILD_INTERNAL) && defined(Q_OS_WIN) && defined(QT_MAKEDLL)
+#    define Q_INTERNAL_EXPORT Q_DECL_EXPORT
+#elif defined(QT_BUILD_INTERNAL) && !defined(Q_OS_WIN) && defined(QT_SHARED)
+#    define Q_INTERNAL_EXPORT Q_DECL_EXPORT
+#else
+#    define Q_INTERNAL_EXPORT
+#endif
+
+/*
+   System information
+*/
 
 class QString;
 class Q_CORE_EXPORT QSysInfo {
@@ -928,7 +974,7 @@ public:
     };
 #if !defined(Q_BYTE_ORDER)
 #  if defined(QT_BUILD_QMAKE)
-    // needed to bootstrap qmake
+    /* needed to bootstrap qmake */
     static const int ByteOrder;
 #  else
 #    error "Qt not configured correctly, please run configure"
@@ -946,6 +992,7 @@ public:
         WV_2000     = 0x0020,
         WV_XP       = 0x0030,
         WV_2003     = 0x0040,
+        WV_VISTA    = 0x0080,
         WV_NT_based = 0x00f0,
 
         WV_CE       = 0x0100,
@@ -958,7 +1005,7 @@ public:
     enum MacVersion {
         MV_Unknown = 0x0000,
 
-        //version
+        /* version */
         MV_9 = 0x0001,
         MV_10_0 = 0x0002,
         MV_10_1 = 0x0003,
@@ -966,7 +1013,7 @@ public:
         MV_10_3 = 0x0005,
         MV_10_4 = 0x0006,
 
-        //codenames
+        /* codenames */
         MV_CHEETAH = MV_10_0,
         MV_PUMA = MV_10_1,
         MV_JAGUAR = MV_10_2,
@@ -1009,7 +1056,7 @@ inline QT3_SUPPORT int qWinVersion() { return QSysInfo::WindowsVersion; }
 #define QT_WA(uni, ansi) ansi
 #define QT_WA_INLINE(uni, ansi) ansi
 #endif
-#endif // Q_WS_WIN
+#endif /* Q_WS_WIN */
 
 #ifndef Q_OUTOFLINE_TEMPLATE
 #  define Q_OUTOFLINE_TEMPLATE
@@ -1022,9 +1069,10 @@ inline QT3_SUPPORT int qWinVersion() { return QSysInfo::WindowsVersion; }
 #  define Q_TYPENAME typename
 #endif
 
-//
-// Use to avoid "unused parameter" warnings
-//
+/*
+   Avoid "unused parameter" warnings
+*/
+
 #if defined(Q_CC_INTEL)
 template <typename T>
 inline void qUnused(T &x) { (void)x; }
@@ -1033,9 +1081,9 @@ inline void qUnused(T &x) { (void)x; }
 #  define Q_UNUSED(x) (void)x;
 #endif
 
-//
-// Debugging and error handling
-//
+/*
+   Debugging and error handling
+*/
 
 #if !defined(QT_NO_DEBUG) && !defined(QT_DEBUG)
 #  define QT_DEBUG
@@ -1045,13 +1093,13 @@ inline void qUnused(T &x) { (void)x; }
 #  define qPrintable(string) (string).toLocal8Bit().constData()
 #endif
 
-Q_CORE_EXPORT void qDebug(const char *, ...) // print debug message
+Q_CORE_EXPORT void qDebug(const char *, ...) /* print debug message */
 #if defined(Q_CC_GNU) && !defined(__INSURE__)
     __attribute__ ((format (printf, 1, 2)))
 #endif
 ;
 
-Q_CORE_EXPORT void qWarning(const char *, ...) // print warning message
+Q_CORE_EXPORT void qWarning(const char *, ...) /* print warning message */
 #if defined(Q_CC_GNU) && !defined(__INSURE__)
     __attribute__ ((format (printf, 1, 2)))
 #endif
@@ -1059,12 +1107,12 @@ Q_CORE_EXPORT void qWarning(const char *, ...) // print warning message
 
 class QString;
 Q_CORE_EXPORT QString qt_error_string(int errorCode = -1);
-Q_CORE_EXPORT void qCritical(const char *, ...) // print critical message
+Q_CORE_EXPORT void qCritical(const char *, ...) /* print critical message */
 #if defined(Q_CC_GNU) && !defined(__INSURE__)
     __attribute__ ((format (printf, 1, 2)))
 #endif
 ;
-Q_CORE_EXPORT void qFatal(const char *, ...) // print fatal message and exit
+Q_CORE_EXPORT void qFatal(const char *, ...) /* print fatal message and exit */
 #if defined(Q_CC_GNU) && !defined(__INSURE__)
     __attribute__ ((format (printf, 1, 2)))
 #endif
@@ -1072,7 +1120,7 @@ Q_CORE_EXPORT void qFatal(const char *, ...) // print fatal message and exit
 
 #ifdef QT3_SUPPORT
 Q_CORE_EXPORT QT3_SUPPORT void qSystemWarning(const char *msg, int code = -1);
-#endif // QT3_SUPPORT
+#endif /* QT3_SUPPORT */
 Q_CORE_EXPORT void qErrnoWarning(int code, const char *msg, ...);
 Q_CORE_EXPORT void qErrnoWarning(const char *msg, ...);
 
@@ -1222,14 +1270,57 @@ inline bool operator!=(QBool b1, bool b2) { return !b1 != !b2; }
 inline bool operator!=(bool b1, QBool b2) { return !b1 != !b2; }
 inline bool operator!=(QBool b1, QBool b2) { return !b1 != !b2; }
 
+static inline bool qFuzzyCompare(double p1, double p2)
+{
+    return qAbs(p1 - p2) < 0.00000000001;
+}
+
+static inline bool qFuzzyCompare(float p1, float p2)
+{
+    return qAbs(p1 - p2) < 0.000001;
+}
+
 /*
- compilers which follow outdated template instantiation rules
- require a class to have a comparison operator to exist when
- a QList of this type is instantiated. It's not actually
- used in the list, though. Hence the dummy implementation.
- Just in case other code relies on it we better trigger a warning
- mandating a real implementation.
+   This function tests a double for a null value. It doesn't
+   check whether the actual value is 0 or close to 0, but whether
+   it is binary 0.
 */
+static inline bool qIsNull(double d)
+{
+    union U {
+        double d;
+        quint64 u;
+    };
+    U val;
+    val.d = d;
+    return val.u == quint64(0);
+}
+
+/*
+   This function tests a float for a null value. It doesn't
+   check whether the actual value is 0 or close to 0, but whether
+   it is binary 0.
+*/
+static inline bool qIsNull(float f)
+{
+    union U {
+        float f;
+        quint32 u;
+    };
+    U val;
+    val.f = f;
+    return val.u == 0u;
+}
+
+/*
+   Compilers which follow outdated template instantiation rules
+   require a class to have a comparison operator to exist when
+   a QList of this type is instantiated. It's not actually
+   used in the list, though. Hence the dummy implementation.
+   Just in case other code relies on it we better trigger a warning
+   mandating a real implementation.
+*/
+
 #ifdef Q_FULL_TEMPLATE_INSTANTIATION
 #  define Q_DUMMY_COMPARISON_OPERATOR(C) \
     bool operator==(const C&) const { \
@@ -1242,8 +1333,8 @@ inline bool operator!=(QBool b1, QBool b2) { return !b1 != !b2; }
 
 
 /*
-  QTypeInfo     - type trait functionality
-  qIsDetached   - data sharing functionality
+   QTypeInfo     - type trait functionality
+   qIsDetached   - data sharing functionality
 */
 
 #ifndef QT_NO_PARTIAL_TEMPLATE_SPECIALIZATION
@@ -1251,6 +1342,7 @@ inline bool operator!=(QBool b1, QBool b2) { return !b1 != !b2; }
 /*
   The catch-all template.
 */
+
 template <typename T> inline bool qIsDetached(T &) { return true; }
 
 template <typename T>
@@ -1299,18 +1391,17 @@ public:
     };
 };
 
-#endif // QT_NO_PARTIAL_TEMPLATE_SPECIALIZATION
-
+#endif /* QT_NO_PARTIAL_TEMPLATE_SPECIALIZATION */
 
 /*
-  Specialize a specific type with:
+   Specialize a specific type with:
 
-    Q_DECLARE_TYPEINFO(type, flags);
+     Q_DECLARE_TYPEINFO(type, flags);
 
-  where 'type' is the name of the type to specialize and 'flags' is
-  logically-OR'ed combination of the flags below.
+   where 'type' is the name of the type to specialize and 'flags' is
+   logically-OR'ed combination of the flags below.
 */
-enum { // TYPEINFO flags
+enum { /* TYPEINFO flags */
     Q_COMPLEX_TYPE = 0,
     Q_PRIMITIVE_TYPE = 0x1,
     Q_STATIC_TYPE = 0,
@@ -1334,19 +1425,19 @@ public: \
 }
 
 /*
-  Specialize a shared type with:
+   Specialize a shared type with:
 
-    Q_DECLARE_SHARED(type);
+     Q_DECLARE_SHARED(type);
 
-  where 'type' is the name of the type to specialize.  NOTE: shared
-  types must declare a 'bool isDetached(void) const;' member for this
-  to work.
+   where 'type' is the name of the type to specialize.  NOTE: shared
+   types must declare a 'bool isDetached(void) const;' member for this
+   to work.
 */
 #define Q_DECLARE_SHARED(TYPE) \
 template <> inline bool qIsDetached<TYPE>(TYPE &t) { return t.isDetached(); }
 
 /*
-  QTypeInfo primitive specializations
+   QTypeInfo primitive specializations
 */
 Q_DECLARE_TYPEINFO(bool, Q_PRIMITIVE_TYPE);
 Q_DECLARE_TYPEINFO(char, Q_PRIMITIVE_TYPE);
@@ -1378,29 +1469,28 @@ Q_CORE_EXPORT void *qMemSet(void *dest, int c, size_t n);
 
 
 /*
-    Avoid some particularly useless warnings from some stupid compilers.
-    To get ALL C++ compiler warnings, define QT_CC_WARNINGS or comment out
-    the line "#define QT_NO_WARNINGS".
+   Avoid some particularly useless warnings from some stupid compilers.
+   To get ALL C++ compiler warnings, define QT_CC_WARNINGS or comment out
+   the line "#define QT_NO_WARNINGS".
 */
-
 #if !defined(QT_CC_WARNINGS)
 #  define QT_NO_WARNINGS
 #endif
 #if defined(QT_NO_WARNINGS)
 #  if defined(Q_CC_MSVC)
-#    pragma warning(disable: 4251) // class 'A' needs to have dll interface for to be used by clients of class 'B'.
-#    pragma warning(disable: 4244) // 'conversion' conversion from 'type1' to 'type2', possible loss of data
-#    pragma warning(disable: 4275) // non - DLL-interface classkey 'identifier' used as base for DLL-interface classkey 'identifier'
-#    pragma warning(disable: 4514) // unreferenced inline/local function has been removed
-#    pragma warning(disable: 4800) // 'type' : forcing value to bool 'true' or 'false' (performance warning)
-#    pragma warning(disable: 4097) // typedef-name 'identifier1' used as synonym for class-name 'identifier2'
-#    pragma warning(disable: 4706) // assignment within conditional expression
-#    pragma warning(disable: 4786) // truncating debug info after 255 characters
-#    pragma warning(disable: 4660) // template-class specialization 'identifier' is already instantiated
-#    pragma warning(disable: 4355) // 'this' : used in base member initializer list
-#    pragma warning(disable: 4231) // nonstandard extension used : 'extern' before template explicit instantiation
-#    pragma warning(disable: 4710) // function not inlined
-#    pragma warning(disable: 4530) // C++ exception handler used, but unwind semantics are not enabled. Specify -GX
+#    pragma warning(disable: 4251) /* class 'A' needs to have dll interface for to be used by clients of class 'B'. */
+#    pragma warning(disable: 4244) /* 'conversion' conversion from 'type1' to 'type2', possible loss of data */
+#    pragma warning(disable: 4275) /* non - DLL-interface classkey 'identifier' used as base for DLL-interface classkey 'identifier' */
+#    pragma warning(disable: 4514) /* unreferenced inline/local function has been removed */
+#    pragma warning(disable: 4800) /* 'type' : forcing value to bool 'true' or 'false' (performance warning) */
+#    pragma warning(disable: 4097) /* typedef-name 'identifier1' used as synonym for class-name 'identifier2' */
+#    pragma warning(disable: 4706) /* assignment within conditional expression */
+#    pragma warning(disable: 4786) /* truncating debug info after 255 characters */
+#    pragma warning(disable: 4660) /* template-class specialization 'identifier' is already instantiated */
+#    pragma warning(disable: 4355) /* 'this' : used in base member initializer list */
+#    pragma warning(disable: 4231) /* nonstandard extension used : 'extern' before template explicit instantiation */
+#    pragma warning(disable: 4710) /* function not inlined */
+#    pragma warning(disable: 4530) /* C++ exception handler used, but unwind semantics are not enabled. Specify -GX */
 #  elif defined(Q_CC_BOR)
 #    pragma option -w-inl
 #    pragma option -w-aus
@@ -1423,8 +1513,6 @@ public:
 inline QFlag::QFlag(int ai) : i(ai) {}
 
 
-//#define Q_NO_TYPESAFE_FLAGS
-
 #ifndef Q_NO_TYPESAFE_FLAGS
 
 template<typename Enum>
@@ -1470,16 +1558,16 @@ inline QFlags<Flags::enum_type> operator|(Flags::enum_type f1, Flags::enum_type
 inline QFlags<Flags::enum_type> operator|(Flags::enum_type f1, QFlags<Flags::enum_type> f2) \
 { return f2 | f1; }
 
-#else // Q_NO_TYPESAFE_FLAGS
+#else /* Q_NO_TYPESAFE_FLAGS */
 
 #define Q_DECLARE_FLAGS(Flags, Enum)\
 typedef uint Flags;
 #define Q_DECLARE_OPERATORS_FOR_FLAGS(Flags)
 
-#endif // Q_NO_TYPESAFE_FLAGS
+#endif /* Q_NO_TYPESAFE_FLAGS */
 
 #if defined(Q_CC_GNU) && !defined(Q_CC_INTEL)
-// make use of typeof-extension
+/* make use of typeof-extension */
 template <typename T>
 class QForeachContainer {
 public:
@@ -1539,7 +1627,7 @@ inline const QForeachContainer<T> *qForeachContainer(const QForeachContainerBase
 #endif
 
 #if 0
-// tell gcc to use its built-in methods for some common functions
+/* tell gcc to use its built-in methods for some common functions */
 #if defined(QT_NO_DEBUG) && defined(Q_CC_GNU)
 #  define qMemCopy __builtin_memcpy
 #  define qMemSet __builtin_memset
@@ -1582,11 +1670,11 @@ Q_CORE_EXPORT QByteArray qgetenv(const char *varName);
 
 inline int qIntCast(double f) { return int(f); }
 inline int qIntCast(float f) { return int(f); }
-typedef double qreal;
 
-//
-// Compat functions that were generated by configure
-//
+
+/*
+   Compat functions that were generated by configure
+*/
 #ifdef QT3_SUPPORT
 #ifndef QT_PRODUCT_LICENSEE
 #  define QT_PRODUCT_LICENSEE QLibraryInfo::licensee()
@@ -1606,21 +1694,23 @@ QT3_SUPPORT Q_CORE_EXPORT const char *qInstallPathSysconf();
 #endif
 
 /*
-  This gives us the possibility to check which modules the user can
-  use. These are purely compile time checks and will generate no code.
+   This gives us the possibility to check which modules the user can
+   use. These are purely compile time checks and will generate no code.
 */
 
-// Qt modules
-#define QT_MODULE_CORE                  0x01
-#define QT_MODULE_GUI                   0x02
-#define QT_MODULE_NETWORK               0x04
-#define QT_MODULE_OPENGL                0x08
-#define QT_MODULE_SQL                   0x10
-#define QT_MODULE_XML                   0x20
-#define QT_MODULE_QT3SUPPORTLIGHT       0x40
-#define QT_MODULE_QT3SUPPORT            0x80
+/* Qt modules */
+#define QT_MODULE_CORE                  0x001
+#define QT_MODULE_GUI                   0x002
+#define QT_MODULE_NETWORK               0x004
+#define QT_MODULE_OPENGL                0x008
+#define QT_MODULE_SQL                   0x010
+#define QT_MODULE_XML                   0x020
+#define QT_MODULE_QT3SUPPORTLIGHT       0x040
+#define QT_MODULE_QT3SUPPORT            0x080
+#define QT_MODULE_SVG                   0x100
+#define QT_MODULE_ACTIVEQT              0x200
 
-// Qt editions
+/* Qt editions */
 #define QT_EDITION_CONSOLE      (QT_MODULE_CORE \
                                  | QT_MODULE_NETWORK \
                                  | QT_MODULE_SQL \
@@ -1628,21 +1718,23 @@ QT3_SUPPORT Q_CORE_EXPORT const char *qInstallPathSysconf();
 #define QT_EDITION_DESKTOPLIGHT (QT_MODULE_CORE \
                                  | QT_MODULE_GUI \
                                  | QT_MODULE_QT3SUPPORTLIGHT)
-#define QT_EDITION_DESKTOP      (QT_MODULE_CORE \
+#define QT_EDITION_OPENSOURCE   (QT_MODULE_CORE \
                                  | QT_MODULE_GUI \
                                  | QT_MODULE_NETWORK \
                                  | QT_MODULE_OPENGL \
                                  | QT_MODULE_SQL \
                                  | QT_MODULE_XML \
                                  | QT_MODULE_QT3SUPPORTLIGHT \
-                                 | QT_MODULE_QT3SUPPORT)
+                                 | QT_MODULE_QT3SUPPORT \
+                                 | QT_MODULE_SVG)
+#define QT_EDITION_DESKTOP      (QT_EDITION_OPENSOURCE \
+                                 | QT_MODULE_ACTIVEQT)
 #define QT_EDITION_UNIVERSAL    QT_EDITION_DESKTOP
 #define QT_EDITION_ACADEMIC     QT_EDITION_DESKTOP
 #define QT_EDITION_EDUCATIONAL  QT_EDITION_DESKTOP
 #define QT_EDITION_EVALUATION   QT_EDITION_DESKTOP
-#define QT_EDITION_OPENSOURCE   QT_EDITION_DESKTOP
 
-// Determine which modules can be used
+/* Determine which modules can be used */
 #ifndef QT_EDITION
 #  ifdef QT_BUILD_QMAKE
 #    define QT_EDITION QT_EDITION_DESKTOP
@@ -1678,6 +1770,12 @@ QT_LICENSED_MODULE(Qt3SupportLight)
 #if (QT_EDITION & QT_MODULE_QT3SUPPORT)
 QT_LICENSED_MODULE(Qt3Support)
 #endif
+#if (QT_EDITION & QT_MODULE_SVG)
+QT_LICENSED_MODULE(Svg)
+#endif
+#if (QT_EDITION & QT_MODULE_ACTIVEQT)
+QT_LICENSED_MODULE(ActiveQt)
+#endif
 
 #define QT_MODULE(x) \
     typedef QtValidLicenseFor##x##Module Qt##x##Module;
