@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -749,7 +749,7 @@ static QByteArray normalizeTypeInternal(const char *t, const char *e, bool fixSc
         } else if (strncmp("long", t+9, 4) == 0
                    // preserve '[unsigned] long long'
                    && (strlen(t + 9 + 4) < 5
-                       || strcmp(t + 9 + 4, " long") != 0
+                       || strncmp(t + 9 + 4, " long", 5) != 0
                       )
                   ) {
             t += 9+4;
@@ -1644,8 +1644,8 @@ QVariant QMetaProperty::read(const QObject *object) const
 
     int  t = QVariant::Int;
     if (!isEnumType()) {
-    int handle = priv(mobj->d.data)->propertyData + 3*idx;
-    int flags = mobj->d.data[handle + 2];
+        int handle = priv(mobj->d.data)->propertyData + 3*idx;
+        int flags = mobj->d.data[handle + 2];
         const char *typeName = mobj->d.stringdata + mobj->d.data[handle + 1];
         t = (flags >> 24);
         if (t == QVariant::Invalid)
@@ -1664,8 +1664,8 @@ QVariant QMetaProperty::read(const QObject *object) const
         argv[0] = value.data();
     }
     const_cast<QObject*>(object)->qt_metacall(QMetaObject::ReadProperty,
-                     idx + mobj->propertyOffset(),
-                     argv);
+                                              idx + mobj->propertyOffset(),
+                                              argv);
     if (t != int(QVariant::LastType) && argv[0] != value.data())
         return QVariant((QVariant::Type)t, argv[0]);
     return value;
@@ -1742,7 +1742,12 @@ bool QMetaProperty::reset(QObject *object) const
     return true;
 }
 
+/*!
+    Returns true if this property can be reset to a default value; otherwise
+    returns false.
 
+    \sa reset()
+*/
 bool QMetaProperty::isResettable() const
 {
     if (!mobj)
@@ -1854,8 +1859,9 @@ bool QMetaProperty::isStored(const QObject *object) const
 }
 
 /*!
-    Returns true if the property is user editable for \a object; otherwise returns
-    false.
+    Returns true if this is the property that the user can editable for \a object;
+    otherwise returns false. I.e. the text property is the user editable property
+    of a QLineEdit.
 
     If no \a object is given, the function returns false if the
     \c{Q_PROPERTY()}'s \c USER attribute is false; otherwise returns
