@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -24,11 +24,11 @@
 #ifndef QRECT_H
 #define QRECT_H
 
-#include "QtCore/qsize.h"
-#include "QtCore/qpoint.h"
+#include <QtCore/qsize.h>
+#include <QtCore/qpoint.h>
 
 #ifdef topLeft
-#error "qrect.h must be included before any header file that defines topLeft"
+#error qrect.h must be included before any header file that defines topLeft
 #endif
 
 QT_MODULE(Core)
@@ -605,7 +605,7 @@ inline QRectF::QRectF(const QRect &r)
 }
 
 inline bool QRectF::isNull() const
-{ return w == 0 && h == 0; }
+{ return qIsNull(w) && qIsNull(h); }
 
 inline bool QRectF::isEmpty() const
 { return w <= 0. || h <= 0.; }
@@ -773,12 +773,14 @@ inline QRectF QRectF::unite(const QRectF &r) const
 
 inline bool operator==(const QRectF &r1, const QRectF &r2)
 {
-    return r1.xp == r2.xp && r1.yp == r2.yp && r1.w == r2.w && r1.h == r2.h;
+    return qFuzzyCompare(r1.xp, r2.xp) && qFuzzyCompare(r1.yp, r2.yp)
+           && qFuzzyCompare(r1.w, r2.w) && qFuzzyCompare(r1.h, r2.h);
 }
 
 inline bool operator!=(const QRectF &r1, const QRectF &r2)
 {
-    return r1.xp != r2.xp || r1.yp != r2.yp || r1.w != r2.w || r1.h != r2.h;
+    return !qFuzzyCompare(r1.xp, r2.xp) || !qFuzzyCompare(r1.yp, r2.yp)
+           || !qFuzzyCompare(r1.w, r2.w) || !qFuzzyCompare(r1.h, r2.h);
 }
 
 inline QRect QRectF::toRect() const
