@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -225,25 +225,38 @@ QPrinter *QAbstractPrintDialog::printer() const
 
     \ingroup dialogs
 
-    It encompasses both the sort of details needed for doing a simple
-    print-out and some print configuration setup.
+    The dialog allows users to change document-related settings, such
+    as the paper size and orientation, type of print (color or
+    grayscale), range of pages, and number of copies to print.
 
-    Typical use of the QPrintDialog is to construct it on a QPrinter
-    object and call exec() to execute it.
+    Controls are also provided to enable users to choose from the
+    printers available, including any configured network printers.
+
+    Typically, QPrintDialog objects are constructed with a QPrinter
+    object, and executed using the exec() function.
 
     \code
-    QPrintDialog printDialog(printer, parent);
-    if (printDialog.exec() == QDialog::Accept) {
-        // print ...
-    }
+        QPrintDialog printDialog(printer, parent);
+        if (printDialog.exec() == QDialog::Accepted) {
+            // print ...
+        }
     \endcode
 
+    If the dialog is accepted by the user, the QPrinter object is
+    correctly configured for printing.
+
     \table
     \row
     \o \inlineimage qprintdialog-plastique.png Screenshot of a Plastique style printer dialog
     \row
     \o A printer dialog in Plastique style
     \endtable
+
+    On Windows and Mac OS X, the native print dialog is used, which
+    means that some QWidget and QDialog properties set on the dialog
+    won't be respected.
+
+    \sa QPageSizeDialog, QPrinter
 */
 
 /*!
