@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -28,6 +28,22 @@
 
 #ifndef QT_NO_TREEWIDGET
 
+/*!
+  \class QTreeWidgetItemIterator
+  \ingroup model-view
+  \brief The QTreeWidgetItemIterator class provides a way to iterate over the 
+  items in a QTreeWidget instance.
+
+  The iterator will walk the items in a pre-order traversal order, thus visiting the parent
+  node \e before it continues to the child nodes.
+
+  It is possible to filter out certain types of node by passing certain
+  \l{IteratorFlag}{flags} to the constructor of QTreeWidgetItemIterator.
+
+
+  \sa QTreeWidget, {Model/View Programming}, QTreeWidgetItem
+*/
+
 /*!
     Constructs an iterator for the same QTreeWidget as \a it. The
     current iterator item is set to point on the current item of \a it.
@@ -41,9 +57,12 @@ QTreeWidgetItemIterator::QTreeWidgetItemIterator(const QTreeWidgetItemIterator &
 }
 
 /*!
-    Constructs an iterator for the QTreeWidget \a widget. The current
-    iterator item is set to point on the first toplevel item (QTreeWidgetItem)
-    of \a widget.
+    Constructs an iterator for the QTreeWidget \a widget. 
+    The iterator is set to point to the first toplevel 
+    item (QTreeWidgetItem) of \a widget or the next matching item if 
+    the toplevel item doesn't match the flags.
+
+    \sa QTreeWidgetItemIterator::IteratorFlag
 */
 
 QTreeWidgetItemIterator::QTreeWidgetItemIterator(QTreeWidget *widget, IteratorFlags flags)
@@ -60,7 +79,7 @@ QTreeWidgetItemIterator::QTreeWidgetItemIterator(QTreeWidget *widget, IteratorFl
 
 /*!
     Constructs an iterator for the QTreeWidget that contains the \a item
-    using the flags \a flags. The current iterator item is set
+    using the flags \a flags. The iterator is set
     to point to \a item or the next matching item if \a item doesn't
     match the flags.
 
@@ -105,9 +124,9 @@ QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator=(const QTreeWidgetIte
 }
 
 /*!
-    Preincrement. Makes the next item the new current item and returns
-    a reference to the iterator.
-    Sets the current pointer to 0 if the current item is the last item.
+    The prefix ++ operator (++it) advances the iterator to the next matching item 
+    and returns a reference to the resulting iterator.
+    Sets the current pointer to 0 if the current item is the last matching item.
 */
 
 QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator++()
@@ -120,9 +139,9 @@ QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator++()
 }
 
 /*!
-    Predecrement. Makes the previous item the new current item and
-    returns a reference to the iterator.
-    Sets the current pointer to 0 if the current item is the first item.
+    The prefix -- operator (--it) advances the iterator to the previous matching item 
+    and returns a reference to the resulting iterator.
+    Sets the current pointer to 0 if the current item is the first matching item.
 */
 
 QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator--()
@@ -177,9 +196,9 @@ bool QTreeWidgetItemIterator::matchesFlags(const QTreeWidgetItem *item) const
     }
 
     {
-        Qt::CheckState check = item->checkState(0); //###FIXME
-        // Not sure why the FIXME is here,
-        // but we (jasaethe && mariusbm) decided that Qt::PartiallyChecked should match as Checked.
+        // ### We only test the check state for column 0
+        Qt::CheckState check = item->checkState(0);
+        // PartiallyChecked matches as Checked.
         if ((flags & Checked) && (check == Qt::Unchecked))
             return false;
         if ((flags & NotChecked) && (check != Qt::Unchecked))
@@ -212,37 +231,38 @@ bool QTreeWidgetItemIterator::matchesFlags(const QTreeWidgetItem *item) const
 }
 
 /*!
-  \fn const QTreeWidgetItemIterator operator++(int)
+  \fn const QTreeWidgetItemIterator QTreeWidgetItemIterator::operator++(int)
 
-  Postincrement. Makes the next item the new current item and returns
-  the iterator for the item that \e was the current item.
+  The postfix ++ operator (it++) advances the iterator to the next matching item 
+  and returns an iterator to the previously current item.
 */
 
 /*!
-  \fn QTreeWidgetItemIterator &operator+=(int n)
+  \fn QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator+=(int n)
 
-  Sets the current item to the item \a n positions after the current
-  item. If that item is beyond the last item, the current item pointer is
-  set to 0. Returns the iterator for the current item.
+  Makes the iterator go forward by n matching items. (If n is negative, the iterator goes backward.)
+
+  If the current item is beyond the last item, the current item pointer is
+  set to 0. Returns the resulting iterator.
 */
 
 /*!
-  \fn const QTreeWidgetItemIterator operator--(int)
+  \fn const QTreeWidgetItemIterator QTreeWidgetItemIterator::operator--(int)
 
-  Postdecrement. Makes the previous item the new current item and
-  returns the iterator for the item that \e was the current item.
+  The postfix -- operator (it--) makes the preceding matching item current and returns an iterator to the previously current item.
 */
 
 /*!
-  \fn QTreeWidgetItemIterator &operator-=(int n)
+  \fn QTreeWidgetItemIterator &QTreeWidgetItemIterator::operator-=(int n)
 
-  Sets the current item to the item \a n positions before the
-  current item. If that item is before the first item, the current
-  item is set to 0. Returns the a reference to the iterator.
+  Makes the iterator go back by n matching items. (If n is negative, the iterator goes forward.)
+
+  If the current item is ahead of the last item, the current item pointer is
+  set to 0. Returns the resulting iterator.
 */
 
 /*!
-  \fn QTreeWidgetItem *operator*() const
+  \fn QTreeWidgetItem *QTreeWidgetItemIterator::operator*() const
 
   Dereference operator. Returns a pointer to the current item.
 */
