@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -668,7 +668,8 @@ QByteArray QX11Data::motifdndObtainData(const char *mimeType)
 {
     QByteArray result;
 
-    Q_ASSERT(Dnd_selection != 0);
+    if (Dnd_selection == 0)
+        return result;
 
     // try to convert the selection to the requested property
     // qDebug("trying to convert to '%s'", mimeType);
