@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -49,6 +49,8 @@
     mainly checks whether there is something that looks like a tag
     before the first line break. Although the result may be correct
     for common cases, there is no guarantee.
+
+    This function is defined in the \c <QTextDocument> header file.
 */
 bool Qt::mightBeRichText(const QString& text)
 {
@@ -103,9 +105,22 @@ bool Qt::mightBeRichText(const QString& text)
 }
 
 /*!
-  Auxiliary function. Converts the plain text string \a plain to a
-  rich text formatted string with any HTML meta-characters escaped.
- */
+    Converts the plain text string \a plain to a HTML string with
+    HTML metacharacters \c{<}, \c{>}, and \c{&} replaced by HTML
+    entities.
+
+    Example:
+
+    \code
+        QString plain = "#include <QtCore>"
+	QString html = Qt::escape(plain);
+	// html == "#include &lt;QtCore&gt;"
+    \endcode
+
+    This function is defined in the \c <QTextDocument> header file.
+
+    \sa convertFromPlainText(), mightBeRichText()
+*/
 QString Qt::escape(const QString& plain)
 {
     QString rich;
@@ -126,15 +141,14 @@ QString Qt::escape(const QString& plain)
 /*!
     \fn QString Qt::convertFromPlainText(const QString &plain, WhiteSpaceMode mode)
 
-    Auxiliary function. Converts the plain text string \a plain to a
-    rich text formatted paragraph while preserving most of its look.
+    Converts the plain text string \a plain to an HTML-formatted
+    paragraph while preserving most of its look.
 
-    \a mode defines the whitespace mode. Possible values are
-    QStyleSheetItem::WhiteSpacePre (no wrapping, all whitespaces
-    preserved) and QStyleSheetItem::WhiteSpaceNormal (wrapping,
-    simplified whitespaces).
+    \a mode defines how whitespace is handled.
 
-    \sa escape()
+    This function is defined in the \c <QTextDocument> header file.
+
+    \sa escape(), mightBeRichText()
 */
 QString Qt::convertFromPlainText(const QString &plain, Qt::WhiteSpaceMode mode)
 {
@@ -186,7 +200,9 @@ QString Qt::convertFromPlainText(const QString &plain, Qt::WhiteSpaceMode mode)
 
 #ifndef QT_NO_TEXTCODEC
 /*!
-  \internal
+    \internal
+
+    This function is defined in the \c <QTextDocument> header file.
 */
 QTextCodec *Qt::codecForHtml(const QByteArray &ba)
 {
@@ -557,7 +573,7 @@ void QTextDocument::setPlainText(const QString &text)
     "<b>bold</b> text" will produce text where the first word has a font
     weight that gives it a bold appearance: "\bold{bold} text".
 
-    \sa setPlainText()
+    \sa setPlainText(), {Supported HTML Subset}
 */
 void QTextDocument::setHtml(const QString &html)
 {
@@ -843,10 +859,10 @@ QFont QTextDocument::defaultFont() const
 
     This signal is emitted whenever the content of the document
     changes in a way that affects the modification state. If \a
-    changed is true if the document has been modified; otherwise it is
+    changed is true, the document has been modified; otherwise it is
     false.
 
-    For example calling setModified(false) on a document and then
+    For example, calling setModified(false) on a document and then
     inserting text causes the signal to get emitted. If you undo that
     operation, causing the document to return to its original
     unmodified state, the signal will get emitted again.
@@ -881,6 +897,11 @@ static void printPage(int index, QPainter *painter, const QTextDocument *doc, co
 
     painter->setClipRect(view);
     ctx.clip = view;
+    
+    // don't use the system palette text as default text color, on HP/UX
+    // for example that's white, and white text on white paper doesn't
+    // look that nice
+    ctx.palette.setColor(QPalette::Text, Qt::black);
 
     layout->draw(painter, ctx);
 
@@ -1294,6 +1315,13 @@ bool QTextHtmlExporter::emitCharFormatStyle(const QTextCharFormat &format)
         attributesEmitted = true;
     }
 
+    if (format.background() != defaultCharFormat.background()) {
+        html += QLatin1String(" background-color:");
+        html += format.background().color().name();
+        html += QLatin1Char(';');
+        attributesEmitted = true;
+    }
+
     if (format.verticalAlignment() != defaultCharFormat.verticalAlignment()) {
         html += QLatin1String(" vertical-align:");
 
@@ -1339,14 +1367,16 @@ void QTextHtmlExporter::emitTextLength(const char *attribute, const QTextLength
         html += QLatin1String("\"");
 }
 
-void QTextHtmlExporter::emitAlignment(Qt::Alignment alignment)
+void QTextHtmlExporter::emitAlignment(Qt::Alignment align)
 {
-    switch (alignment & Qt::AlignHorizontal_Mask) {
-        case Qt::AlignLeft: break;
-        case Qt::AlignRight: html += QLatin1String(" align='right'"); break;
-        case Qt::AlignHCenter: html += QLatin1String(" align='center'"); break;
-        case Qt::AlignJustify: html += QLatin1String(" align='justify'"); break;
-    }
+    if (align & Qt::AlignLeft)
+        return;
+    else if (align & Qt::AlignRight)
+        html += QLatin1String(" align=\"right\"");
+    else if (align & Qt::AlignHCenter)
+        html += QLatin1String(" align=\"center\"");
+    else if (align & Qt::AlignJustify)
+        html += QLatin1String(" align=\"justify\"");
 }
 
 void QTextHtmlExporter::emitFloatStyle(QTextFrameFormat::Position pos, StyleMode mode)
@@ -1789,6 +1819,8 @@ void QTextHtmlExporter::emitFrame(QTextFrame::Iterator frameIt)
     If you later on convert the returned html string into a byte array for
     transmission over a network or when saving to disk you should specify
     the encoding you're going to use for the conversion to a byte array here.
+
+    \sa {Supported HTML Subset}
 */
 QString QTextDocument::toHtml(const QByteArray &encoding) const
 {
