@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -556,8 +556,10 @@ void QTextHtmlImporter::import()
 
             if (node->isTableCell && !tables.isEmpty()) {
                 Table &t = tables.last();
-                if (t.table) {
-                    cursor.setPosition(t.currentPosition.cell().firstPosition());
+                if (!t.isTextFrame) {
+                    const QTextTableCell cell = t.currentCell.cell();
+                    if (cell.isValid())
+                        cursor.setPosition(cell.firstPosition());
                 }
                 hasBlock = true;
 
@@ -698,32 +700,36 @@ bool QTextHtmlImporter::closeTag(int i)
         if (closedNode->id == Html_tr && !tables.isEmpty()) {
             Table &t = tables.last();
 
-            if (t.table) {
+            if (!t.isTextFrame) {
                 ++t.currentRow;
-                while (!t.currentPosition.atEnd() && t.currentPosition.row < t.currentRow)
-                    ++t.currentPosition;
-
-                t.currentRow = t.currentPosition.row;
+                
+                // for broken html with rowspans but missing tr tags
+                while (!t.currentCell.atEnd() && t.currentCell.row < t.currentRow)
+                    ++t.currentCell;
             }
 
             blockTagClosed = true;
         } else if (closedNode->id == Html_table && !tables.isEmpty()) {
-            Table &t = tables.last();
-            if (QTextTable *parentTable = qobject_cast<QTextTable *>(t.lastFrame)) {
-                cursor = parentTable->cellAt(t.lastRow, t.lastColumn).lastCursorPosition();
+            tables.resize(tables.size() - 1);
+
+            if (tables.isEmpty()) {
+                cursor = doc->rootFrame()->lastCursorPosition();
             } else {
-                cursor = t.lastFrame->lastCursorPosition();
+                Table &t = tables.last();
+                if (t.isTextFrame)
+                    cursor = t.frame->lastCursorPosition();
+                else
+                    cursor = t.currentCell.cell().lastCursorPosition();
             }
 
-            tables.resize(tables.size() - 1);
             // we don't need an extra block after tables, so we don't
             // claim to have closed one for the creation of a new one
             // in import()
             blockTagClosed = false;
         } else if (closedNode->isTableCell && !tables.isEmpty()) {
             Table &t = tables.last();
-            if (t.table)
-                ++tables.last().currentPosition;
+            if (!t.isTextFrame)
+                ++tables.last().currentCell;
             blockTagClosed = true;
         } else if (closedNode->isListStart) {
 
@@ -742,6 +748,11 @@ bool QTextHtmlImporter::closeTag(int i)
                    || closedNode->id == Html_h6
                   ) {
             blockTagClosed = true;
+        } else if (closedNode->id == Html_p) {
+            // blockTagClosed may result in the creation of a
+            // new block
+            if (!closedNode->text.isEmpty())
+                blockTagClosed = true;
         }
 
         closedNode = &at(closedNode->parent);
@@ -755,38 +766,52 @@ QTextHtmlImporter::Table QTextHtmlImporter::scanTable(int tableNodeIdx)
 {
     Table table;
     table.columns = 0;
-    table.lastFrame = cursor.currentFrame();
 
     QVector<QTextLength> columnWidths;
     QVector<int> rowSpanCellsPerRow;
 
+    QVector<int> rowNodes;
+    rowNodes.reserve(at(tableNodeIdx).children.count());
+    foreach (int row, at(tableNodeIdx).children)
+        switch (at(row).id) {
+            case Html_tr:
+                rowNodes += row;
+                break;
+            case Html_thead:
+            case Html_tbody:
+            case Html_tfoot:
+                foreach (int potentialRow, at(row).children)
+                    if (at(potentialRow).id == Html_tr)
+                        rowNodes += potentialRow;
+                break;
+            default: break;
+        }
+
     int effectiveRow = 0;
-    foreach (int row, at(tableNodeIdx).children) {
-        if (at(row).id == Html_tr) {
-            int colsInRow = 0;
+    foreach (int row, rowNodes) {
+        int colsInRow = 0;
 
-            foreach (int cell, at(row).children)
-                if (at(cell).isTableCell) {
+        foreach (int cell, at(row).children)
+            if (at(cell).isTableCell) {
 
-                    const QTextHtmlParserNode &c = at(cell);
-                    colsInRow += c.tableCellColSpan;
+                const QTextHtmlParserNode &c = at(cell);
+                colsInRow += c.tableCellColSpan;
 
-                    if (c.tableCellRowSpan > 1) {
-                        rowSpanCellsPerRow.resize(effectiveRow + c.tableCellRowSpan + 1);
+                if (c.tableCellRowSpan > 1) {
+                    rowSpanCellsPerRow.resize(effectiveRow + c.tableCellRowSpan + 1);
 
-                        for (int r = effectiveRow + 1; r < effectiveRow + c.tableCellRowSpan; ++r)
-                            rowSpanCellsPerRow[r]++;
-                    }
-
-                    while (columnWidths.count() < colsInRow)
-                        columnWidths << c.width;
+                    for (int r = effectiveRow + 1; r < effectiveRow + c.tableCellRowSpan; ++r)
+                        rowSpanCellsPerRow[r]++;
                 }
 
-            table.columns = qMax(table.columns, colsInRow + rowSpanCellsPerRow.value(effectiveRow, 0));
+                while (columnWidths.count() < colsInRow)
+                    columnWidths << c.width;
+            }
 
-            ++effectiveRow;
-            rowSpanCellsPerRow.append(0);
-        }
+        table.columns = qMax(table.columns, colsInRow + rowSpanCellsPerRow.value(effectiveRow, 0));
+
+        ++effectiveRow;
+        rowSpanCellsPerRow.append(0);
     }
     table.rows = effectiveRow;
 
@@ -795,7 +820,7 @@ QTextHtmlImporter::Table QTextHtmlImporter::scanTable(int tableNodeIdx)
 
     QTextFrameFormat fmt;
     const QTextHtmlParserNode &node = at(tableNodeIdx);
-    if (node.isTableFrame) {
+    if (node.isTextFrame) {
         // for plain text frames we set the frame margin
         // for all of top/bottom/left/right, so in the import 
         // here it doesn't matter which one we pick
@@ -823,32 +848,26 @@ QTextHtmlImporter::Table QTextHtmlImporter::scanTable(int tableNodeIdx)
         fmt.clearBackground();
     fmt.setPosition(QTextFrameFormat::Position(node.cssFloat));
 
-    table.lastFrame = cursor.currentFrame();
-    if (QTextTable *parentTable = qobject_cast<QTextTable *>(table.lastFrame)) {
-        QTextTableCell cell = parentTable->cellAt(cursor);
-        table.lastRow = cell.row();
-        table.lastColumn = cell.column();
-    }
-
-    if (node.isTableFrame) {
-        cursor.insertFrame(fmt);
+    if (node.isTextFrame) {
+        table.frame = cursor.insertFrame(fmt);
+        table.isTextFrame = true;
     } else {
-        table.table = cursor.insertTable(table.rows, table.columns, fmt.toTableFormat());
+        QTextTable *textTable = cursor.insertTable(table.rows, table.columns, fmt.toTableFormat());
+        table.frame = textTable;
 
-        TableIterator it(table.table);
-        foreach (int row, at(tableNodeIdx).children)
-            if (at(row).id == Html_tr)
-                foreach (int cell, at(row).children)
-                    if (at(cell).isTableCell) {
-                        const QTextHtmlParserNode &c = at(cell);
+        TableCellIterator it(textTable);
+        foreach (int row, rowNodes)
+            foreach (int cell, at(row).children)
+            if (at(cell).isTableCell) {
+                const QTextHtmlParserNode &c = at(cell);
 
-                        if (c.tableCellColSpan > 1 || c.tableCellRowSpan > 1)
-                            table.table->mergeCells(it.row, it.column, c.tableCellRowSpan, c.tableCellColSpan);
+                if (c.tableCellColSpan > 1 || c.tableCellRowSpan > 1)
+                    textTable->mergeCells(it.row, it.column, c.tableCellRowSpan, c.tableCellColSpan);
 
-                        ++it;
-                    }
+                ++it;
+            }
 
-        table.currentPosition = TableIterator(table.table);
+        table.currentCell = TableCellIterator(textTable);
     }
     return table;
 }
