@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -35,14 +35,13 @@
 // We mean it.
 //
 
-#include <qvector.h>
-#include <qcolor.h>
-#include <qfont.h>
-#include <private/qtextformat_p.h>
-#include <private/qtextdocument_p.h>
-
-#include "qtextdocument.h"
-#include "qtextcursor.h"
+#include "QtCore/qvector.h"
+#include "QtGui/qcolor.h"
+#include "QtGui/qfont.h"
+#include "QtGui/qtextdocument.h"
+#include "QtGui/qtextcursor.h"
+#include "private/qtextformat_p.h"
+#include "private/qtextdocument_p.h"
 
 enum QTextHTMLElements {
     Html_qt,
@@ -104,6 +103,9 @@ enum QTextHTMLElements {
     Html_tr,
     Html_td,
     Html_th,
+    Html_thead,
+    Html_tbody,
+    Html_tfoot,
     Html_html,
 
     // misc...
@@ -118,7 +120,7 @@ struct QTextHtmlElement
 {
     const char *name;
     int id;
-    enum DisplayMode { DisplayBlock, DisplayInline, DisplayNone } displayMode;
+    enum DisplayMode { DisplayBlock, DisplayInline, DisplayTable, DisplayNone } displayMode;
 };
 
 class QTextHtmlParser;
@@ -142,6 +144,8 @@ struct QTextHtmlParserAttribute {
 };
 Q_DECLARE_TYPEINFO(QTextHtmlParserAttribute, Q_MOVABLE_TYPE);
 
+enum QTriState { Off = 0, On = 1, Unspecified = 2 };
+
 struct QTextHtmlParserNode {
     enum WhiteSpaceMode {
         WhiteSpaceNormal,
@@ -163,23 +167,25 @@ struct QTextHtmlParserNode {
     uint isListStart : 1;
     uint isTableCell : 1;
     uint isAnchor : 1;
-    uint fontItalic : 1;
-    uint fontUnderline : 1;
-    uint fontOverline : 1;
-    uint fontStrikeOut : 1;
-    uint fontFixedPitch : 1;
+    uint fontItalic : 2; // Tristate
+    uint fontUnderline : 2; // Tristate
+    uint fontOverline : 2; // Tristate
+    uint fontStrikeOut : 2; // Tristate
+    uint fontFixedPitch : 2; // Tristate
     uint cssFloat : 2;
     uint hasOwnListStyle : 1;
     uint hasFontPointSize : 1;
+    uint hasFontPixelSize : 1;
     uint hasFontSizeAdjustment : 1;
     uint hasCssBlockIndent : 1;
     uint hasCssListIndent : 1;
     uint isEmptyParagraph : 1;
-    uint isTableFrame : 1;
+    uint isTextFrame : 1;
     uint direction : 2; // 3 means unset
     uint displayMode : 3; // QTextHtmlElement::DisplayMode
     QString fontFamily;
     int fontPointSize;
+    int fontPixelSize;
     int fontSizeAdjustment;
     int fontWeight;
     QColor color;
@@ -217,7 +223,11 @@ struct QTextHtmlParserNode {
         switch (id) {
             case Html_dd: return (parentId == Html_dt || parentId == Html_dl);
             case Html_dt: return (parentId == Html_dl);
-            case Html_tr: return (parentId == Html_table);
+            case Html_tr: return (parentId == Html_table
+                                  || parentId == Html_thead
+                                  || parentId == Html_tbody
+                                  || parentId == Html_tfoot
+                                 );
             case Html_th:
             case Html_td: return (parentId == Html_tr);
             default: break;
@@ -232,7 +242,6 @@ struct QTextHtmlParserNode {
 
     inline int uncollapsedMargin(int mar) const { return margin[mar]; }
 
-private:
     bool isNestedList(const QTextHtmlParser *parser) const;
 
     int margin[4];
