@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
@@ -1545,12 +1545,6 @@ QHttp::~QHttp()
     abort();
 }
 
-/*!
-    \fn int QHttp::supportedOperations() const
-
-    \internal
-*/
-
 /*!
     \enum QHttp::State
 
