@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -39,4 +39,4 @@ public:
     bool save(const QString &fileName, const char *format = 0);
 };
 
-#endif
+#endif // Q3PICTURE_H
