@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -24,7 +24,7 @@
 #ifndef Q3BUTTON_H
 #define Q3BUTTON_H
 
-#include "QtGui/qabstractbutton.h"
+#include <QtGui/qabstractbutton.h>
 
 QT_MODULE(Qt3SupportLight)
 
@@ -42,4 +42,4 @@ protected:
 
 };
 
-#endif
+#endif // Q3BUTTON_H
