@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -24,11 +24,12 @@
 #ifndef Q3GROUPBOX_H
 #define Q3GROUPBOX_H
 
-#include "QtGui/qgroupbox.h"
+#include <QtGui/qgroupbox.h>
 
 QT_MODULE(Qt3SupportLight)
 
 class Q3GroupBoxPrivate;
+
 class Q_COMPAT_EXPORT Q3GroupBox : public QGroupBox
 {
     Q_OBJECT
