@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtSVG module of the Qt Toolkit.
 **
@@ -34,6 +34,7 @@
 #include "qpen.h"
 #include "qpainterpath.h"
 #include "qbrush.h"
+#include "qcolor.h"
 #include "qtextformat.h"
 #include "qvector.h"
 #include "qdebug.h"
@@ -47,7 +48,7 @@
 
 static bool parsePathDataFast(const QString &data, QPainterPath &path);
 
-static QPen defaultPen(Qt::black, 1, Qt::SolidLine,
+static QPen defaultPen(Qt::black, 1, Qt::NoPen,
                        Qt::FlatCap, Qt::MiterJoin);
 
 static QString xmlSimplify(const QString &str)
@@ -81,7 +82,7 @@ static QList<qreal> parseNumbersList(QString::const_iterator &itr)
     while ((*itr).isSpace())
         ++itr;
     while ((*itr).isNumber() ||
-           (*itr) == '-' || (*itr) == '+') {
+           (*itr) == '-' || (*itr) == '+' || (*itr) == '.') {
         temp = QString();
 
         if ((*itr) == '-')
@@ -174,7 +175,7 @@ static QString idFromUrl(const QString &url)
  * returns true when successfuly set the color. false signifies
  * that the color should be inherited
  */
-static bool resolveColor(const QString &colorStr, QColor &color)
+static bool resolveColor(const QString &colorStr, QColor &color, QSvgHandler *handler)
 {
     static QHash<QString, QColor> colors;
     QString colorStrTr = colorStr.trimmed();
@@ -219,9 +220,11 @@ static bool resolveColor(const QString &colorStr, QColor &color)
                        int(compo[2]));
         return true;
     } else if (colorStr == QLatin1String("inherited") ||
-               colorStr == QLatin1String("inherit") ||
-               colorStr == QLatin1String("currentColor")) {
+               colorStr == QLatin1String("inherit"))  {
         return false;
+    } else if (colorStr == QLatin1String("currentColor")) {
+        color = handler->currentColor();
+        return true;
     }
 
     color = QColor(colorStrTr);
@@ -229,9 +232,9 @@ static bool resolveColor(const QString &colorStr, QColor &color)
 }
 
 static bool constructColor(const QString &colorStr, const QString &opacity,
-                           QColor &color)
+                           QColor &color, QSvgHandler *handler)
 {
-    if (!resolveColor(colorStr, color))
+    if (!resolveColor(colorStr, color, handler))
         return false;
     if (!opacity.isEmpty()) {
         qreal op = opacity.toDouble();
@@ -302,7 +305,8 @@ static bool createSvgGlyph(QSvgFont *font, const QXmlAttributes &attributes)
     return true;
 }
 
-
+// this should really be called convertToDefaultCoordinateSystem
+// and convert when type != QSvgHandler::defaultCoordinateSystem
 static qreal convertToPixels(qreal len, bool isX, QSvgHandler::LengthType type)
 {
     QWidgetList widgets = QApplication::topLevelWidgets();
@@ -345,26 +349,21 @@ static qreal convertToPixels(qreal len, bool isX, QSvgHandler::LengthType type)
     return len;
 }
 
-static void parseColor(QSvgNode *node,
+static void parseColor(QSvgNode *,
                        const QXmlAttributes &attributes,
-                       QSvgHandler *)
+                       QSvgHandler *handler)
 {
     QString colorStr = attributes.value("color");
     QString opacity  = attributes.value("color-opacity");
     QColor color;
-    if (constructColor(colorStr, opacity, color)) {
-        QSvgStyleProperty *prop = new QSvgFillStyle(QBrush(color), true);
-        node->appendStyleProperty(prop, QString());
-        // SVG 1.1 Conformance test painting-fill-02-t.svg
-        // makes it seem color only sets the fill not the stroke
-        //prop = new QSvgStrokeStyle(QPen(color));
-        //node->appendStyleProperty(prop, QString());
+    if (constructColor(colorStr, opacity, color, handler)) {
+        handler->pushColor(color);
     }
 }
 
 static void parseBrush(QSvgNode *node,
                        const QXmlAttributes &attributes,
-                       QSvgHandler *)
+                       QSvgHandler *handler)
 {
     QString value = attributes.value("fill");
     QString myId = attributes.value("id");
@@ -396,8 +395,11 @@ static void parseBrush(QSvgNode *node,
         } else if (value != QLatin1String("none")) {
             QString opacity = attributes.value("fill-opacity");
             QString fillRule = attributes.value("fill-rule");
+
+            if (opacity.isEmpty())
+                opacity = attributes.value("opacity");
             QColor color;
-            if (constructColor(value, opacity, color)) {
+            if (constructColor(value, opacity, color, handler)) {
                 QSvgStyleProperty *prop = new QSvgFillStyle(QBrush(color));
                 node->appendStyleProperty(prop, myId);
             }
@@ -411,7 +413,7 @@ static void parseBrush(QSvgNode *node,
 
 static void parseQPen(QPen &pen, QSvgNode *node,
                       const QXmlAttributes &attributes,
-                      QSvgHandler *)
+                      QSvgHandler *handler)
 {
     QString value = attributes.value("stroke");
     QString dashArray  = attributes.value("stroke-dasharray");
@@ -422,6 +424,10 @@ static void parseQPen(QPen &pen, QSvgNode *node,
     QString opacity    = attributes.value("stroke-opacity");
     QString width      = attributes.value("stroke-width");
     QString myId       = attributes.value("id");
+
+    if (opacity.isEmpty())
+        opacity = attributes.value("opacity");
+
     if (!value.isEmpty() || !width.isEmpty()) {
         if (value != QLatin1String("none")) {
             if (!value.isEmpty()) {
@@ -444,7 +450,7 @@ static void parseQPen(QPen &pen, QSvgNode *node,
                         if (style->type() == QSvgStyleProperty::GRADIENT) {
                             QBrush b(*((QSvgGradientStyle*)style)->qgradient());
                             pen.setBrush(b);
-                        } else if (style->type() == QSvgStyleProperty::GRADIENT) {
+                        } else if (style->type() == QSvgStyleProperty::SOLID_COLOR) {
                             pen.setColor(
                                 ((QSvgSolidColorStyle*)style)->qcolor());
                         }
@@ -453,7 +459,7 @@ static void parseQPen(QPen &pen, QSvgNode *node,
                     }
                 } else {
                     QColor color;
-                    if (constructColor(value, opacity, color))
+                    if (constructColor(value, opacity, color, handler))
                         pen.setColor(color);
                 }
                 //since we could inherit stroke="none"
@@ -461,9 +467,14 @@ static void parseQPen(QPen &pen, QSvgNode *node,
                 pen.setStyle(Qt::SolidLine);
             }
             if (!width.isEmpty()) {
-                pen.setWidthF(width.toDouble());
-//                 if (miterlimit.isEmpty())
-//                     pen.setMiterLimit(pen.miterLimit()/pen.widthF());
+                QSvgHandler::LengthType lt;
+                qreal widthF = parseLength(width, lt, handler);
+                //### fixme
+                if (!widthF) {
+                    pen.setStyle(Qt::NoPen);
+                    return;
+                }
+                pen.setWidthF(widthF);
             }
             qreal penw = pen.widthF();
 
@@ -476,7 +487,7 @@ static void parseQPen(QPen &pen, QSvgNode *node,
                     pen.setJoinStyle(Qt::BevelJoin);
             }
             if (!miterlimit.isEmpty()) {
-                pen.setMiterLimit(miterlimit.toDouble()/penw);
+                pen.setMiterLimit(miterlimit.toDouble()/2);
             }
 
             if (!linecap.isEmpty()) {
@@ -614,7 +625,7 @@ static QMatrix parseTransformationMatrix(const QString &value)
 
 static void parsePen(QSvgNode *node,
                      const QXmlAttributes &attributes,
-                     QSvgHandler *)
+                     QSvgHandler *handler)
 {
     QString value = attributes.value("stroke");
     QString dashArray  = attributes.value("stroke-dasharray");
@@ -625,7 +636,12 @@ static void parsePen(QSvgNode *node,
     QString opacity    = attributes.value("stroke-opacity");
     QString width      = attributes.value("stroke-width");
     QString myId       = attributes.value("id");
-    if (!value.isEmpty() || !width.isEmpty()) {
+
+    if (opacity.isEmpty())
+        opacity = attributes.value("opacity");
+
+    if (!value.isEmpty() || !width.isEmpty() || !linecap.isEmpty() ||
+        linejoin.isEmpty()) {
         if (value != QLatin1String("none")) {
             QSvgStrokeStyle *inherited =
                 static_cast<QSvgStrokeStyle*>(node->parent()->styleProperty(
@@ -663,7 +679,7 @@ static void parsePen(QSvgNode *node,
                     }
                 } else {
                     QColor color;
-                    if (constructColor(value, opacity, color))
+                    if (constructColor(value, opacity, color, handler))
                         pen.setColor(color);
                 }
                 //since we could inherit stroke="none"
@@ -671,9 +687,14 @@ static void parsePen(QSvgNode *node,
                 pen.setStyle(Qt::SolidLine);
             }
             if (!width.isEmpty()) {
-                pen.setWidthF(width.toDouble());
-                if (miterlimit.isEmpty())
-                    pen.setMiterLimit(pen.miterLimit()/pen.widthF());
+                QSvgHandler::LengthType lt;
+                qreal widthF = parseLength(width, lt, handler);
+                //### fixme
+                if (!widthF) {
+                    pen.setStyle(Qt::NoPen);
+                    return;
+                }
+                pen.setWidthF(widthF);
             }
 
             if (!linejoin.isEmpty()) {
@@ -708,7 +729,7 @@ static void parsePen(QSvgNode *node,
                 pen.setDashPattern(vec);
             }
             if (!miterlimit.isEmpty()) {
-                pen.setMiterLimit(miterlimit.toDouble()/penw);
+                pen.setMiterLimit(miterlimit.toDouble() / 2);
             }
 
             node->appendStyleProperty(new QSvgStrokeStyle(pen), myId);
@@ -722,10 +743,14 @@ static void parsePen(QSvgNode *node,
 
 
 static bool parseQBrush(const QXmlAttributes &attributes, QSvgNode *node,
-                        QBrush &brush, QSvgHandler *)
+                        QBrush &brush, QSvgHandler *handler)
 {
     QString value = attributes.value("fill");
     QString opacity = attributes.value("fill-opacity");
+
+    if (opacity.isEmpty())
+        opacity = attributes.value("opacity");
+
     QColor color;
     if (!value.isEmpty() || !opacity.isEmpty()) {
         if (value.startsWith("url")) {
@@ -766,7 +791,7 @@ static bool parseQBrush(const QXmlAttributes &attributes, QSvgNode *node,
                 }
             }
         } else if (value != QLatin1String("none")) {
-            if (constructColor(value, opacity, color)) {
+            if (constructColor(value, opacity, color, handler)) {
                 brush.setStyle(Qt::SolidPattern);
                 brush.setColor(color);
             }
@@ -796,6 +821,9 @@ static bool parseQFont(const QXmlAttributes &attributes,
             QSvgHandler::LengthType type;
             qreal len = parseLength(size, type, handler);
             //len = convertToPixels(len, false, type);
+            // ### org_module.svg shows that font size
+            // seems to be always in px...
+            type  = QSvgHandler::PX;
             if (type == QSvgHandler::PX ||
                 type == QSvgHandler::OTHER)
                 font.setPixelSize(int(len));
@@ -872,6 +900,7 @@ static void parseFont(QSvgNode *node,
         font = inherited->qfont();
     if (parseQFont(attributes, font, handler)) {
         QString myId = attributes.value("id");
+        QString anchor = attributes.value("text-anchor");
         QSvgTinyDocument *doc = node->document();
         QSvgFontStyle *fontStyle = 0;
         QString family = (font.family().isEmpty())?myId:font.family();
@@ -887,6 +916,8 @@ static void parseFont(QSvgNode *node,
         }
         if (!fontStyle)
             fontStyle = new QSvgFontStyle(font);
+        if (!anchor.isEmpty())
+            fontStyle->setTextAnchor(anchor);
 
         node->appendStyleProperty(fontStyle, myId);
     }
@@ -1338,26 +1369,29 @@ static bool parseDefaultTextStyle(QSvgNode *node,
     QXmlAttributes attrs = attributes;
     QString css = attrs.value("style");
     QString fontFamily = attrs.value("font-family");
+
     parseCSStoXMLAttrs(css, attrs);
 
+    QString anchor = attrs.value("text-anchor");
+
     QSvgFontStyle *fontStyle = static_cast<QSvgFontStyle*>(
         node->styleProperty(QSvgStyleProperty::FONT));
     if (fontStyle) {
         QSvgTinyDocument *doc = fontStyle->doc();
         if (doc && fontStyle->svgFont()) {
-            parseStyle(node, attributes, handler);
+            parseStyle(node, attrs, handler);
             return true;
         }
     } else if (!fontFamily.isEmpty()) {
         QSvgTinyDocument *doc = node->document();
         QSvgFont *svgFont = doc->svgFont(fontFamily);
         if (svgFont) {
-            parseStyle(node, attributes, handler);
+            parseStyle(node, attrs, handler);
             return true;
         }
     }
 
-    QString anchor = attrs.value("text-anchor");
+
     QTextCharFormat format;
     QFont font;
     QBrush brush(QColor(0, 0, 0));
@@ -1368,8 +1402,11 @@ static bool parseDefaultTextStyle(QSvgNode *node,
     if (initial) {
         QSvgFontStyle *fontStyle = static_cast<QSvgFontStyle*>(
             node->parent()->styleProperty(QSvgStyleProperty::FONT));
-        if (fontStyle)
+        if (fontStyle) {
             font = fontStyle->qfont();
+            if (anchor.isEmpty())
+                anchor = fontStyle->textAnchor();
+        }
     }
     if (parseQFont(attrs, font, handler) || initial) {
         if (font.pixelSize() != -1)
@@ -1380,7 +1417,7 @@ static bool parseDefaultTextStyle(QSvgNode *node,
     if (initial) {
         QSvgFillStyle *fillStyle = static_cast<QSvgFillStyle*>(
             node->styleProperty(QSvgStyleProperty::FILL));
-        if (fillStyle  && !fillStyle->fromColor()) {
+        if (fillStyle) {
             brush = fillStyle->qbrush();
         }
     }
@@ -1521,7 +1558,7 @@ static bool parseAnimateNode(QSvgNode *parent,
 
 static bool parseAnimateColorNode(QSvgNode *parent,
                                   const QXmlAttributes &attributes,
-                                  QSvgHandler *)
+                                  QSvgHandler *handler)
 {
     QString typeStr    = attributes.value("type");
     QString fromStr    = attributes.value("from");
@@ -1536,8 +1573,8 @@ static bool parseAnimateColorNode(QSvgNode *parent,
     QList<QColor> colors;
     if (valuesStr.isEmpty()) {
         QColor startColor, endColor;
-        constructColor(fromStr, QString(), startColor);
-        constructColor(toStr, QString(), endColor);
+        constructColor(fromStr, QString(), startColor, handler);
+        constructColor(toStr, QString(), endColor, handler);
         colors.append(startColor);
         colors.append(endColor);
     } else {
@@ -1545,7 +1582,7 @@ static bool parseAnimateColorNode(QSvgNode *parent,
         QStringList::const_iterator itr;
         for (itr = str.constBegin(); itr != str.constEnd(); ++itr) {
             QColor color;
-            constructColor(*itr, QString(), color);
+            constructColor(*itr, QString(), color, handler);
             colors.append(color);
         }
     }
@@ -2232,13 +2269,17 @@ static bool parseSetNode(QSvgNode *parent,
 
 static QSvgStyleProperty *createSolidColorNode(QSvgNode *parent,
                                                const QXmlAttributes &attributes,
-                                               QSvgHandler *)
+                                               QSvgHandler *handler)
 {
     Q_UNUSED(parent); Q_UNUSED(attributes);
     QString solidColorStr = attributes.value("solid-color");
     QString solidOpacityStr = attributes.value("solid-opacity");
+
+    if (solidOpacityStr.isEmpty())
+        solidOpacityStr = attributes.value("opacity");
+
     QColor color;
-    if (!constructColor(solidColorStr, solidOpacityStr, color))
+    if (!constructColor(solidColorStr, solidOpacityStr, color, handler))
         return 0;
     QSvgSolidColorStyle *style = new QSvgSolidColorStyle(color);
     return style;
@@ -2269,7 +2310,7 @@ static bool parseStopNode(QSvgStyleProperty *parent,
     if (type == QSvgHandler::PERCENT) {
         offset = offset/100.0;
     }
-    bool colorOK = constructColor(colorStr, opacityStr, color);
+    bool colorOK = constructColor(colorStr, opacityStr, color, handler);
     QGradient *grad = style->qgradient();
     //qDebug()<<"set color at"<<offset<<color;
     grad->setColorAt(offset, color);
@@ -2341,10 +2382,12 @@ static QSvgNode *createSvgNode(QSvgNode *parent,
     }
 
 
-    if (type == QSvgHandler::PT)
+    if (type == QSvgHandler::PT) {
         handler->setDefaultCoordinateSystem(type);
-    else
+    }
+    else {
         handler->setDefaultCoordinateSystem(QSvgHandler::PX);
+    }
 
     return node;
 }
@@ -2375,9 +2418,13 @@ static QSvgNode *createTextNode(QSvgNode *parent,
     //### editable and rotate not handled
     QSvgHandler::LengthType type;
     qreal nx = parseLength(x, type, handler);
-    nx = convertToPixels(nx, true, type);
     qreal ny = parseLength(y, type, handler);
-    ny = convertToPixels(ny, true, type);
+
+    //### not to pixels but to the default coordinate system
+    //    and text should be already in the correct coordinate
+    //    system here
+    //nx = convertToPixels(nx, true, type);
+    //ny = convertToPixels(ny, true, type);
 
     QSvgNode *text = new QSvgText(parent, QPointF(nx, ny));
     return text;
@@ -2459,7 +2506,7 @@ QSvgHandler::QSvgHandler()
     : m_doc(0), m_style(0), m_defaultCoords(PX)
 {
     if (s_groupFactory.isEmpty()) {
-        defaultPen.setMiterLimit(4);
+        defaultPen.setMiterLimit(2);
         init();
     }
 }
@@ -2472,6 +2519,13 @@ bool QSvgHandler::startElement(const QString &namespaceURI,
     Q_UNUSED(namespaceURI);
     QSvgNode *node = 0;
     //qDebug()<<"localName = "<<localName;
+
+    if (m_colorTagCount.count()) {
+        int top = m_colorTagCount.pop();
+        ++top;
+        m_colorTagCount.push(top);
+    }
+
     if (s_groupFactory.contains(localName)) {
         //group
         m_style = 0;
@@ -2537,6 +2591,8 @@ bool QSvgHandler::startElement(const QString &namespaceURI,
             m_nodes.top(), attributes, this);
         if (prop) {
             QString id = attributes.value("id");
+            if (id.isEmpty())
+                id = attributes.value("xml:id");
             m_nodes.top()->appendStyleProperty(prop, id, true);
             m_style = prop;
         } else {
@@ -2572,6 +2628,16 @@ bool QSvgHandler::endElement(const QString &namespaceURI,
     CurrentNode node = m_skipNodes.top();
     m_skipNodes.pop();
 
+    if (m_colorTagCount.count()) {
+        int top = m_colorTagCount.pop();
+        --top;
+        if (!top) {
+            m_colorStack.pop();
+        } else {
+            m_colorTagCount.push(top);
+        }
+    }
+
     if (node == Unknown) {
         //qDebug()<<"Skipping "<< qName<< ":"<<namespaceURI;
         return true;
@@ -2690,3 +2756,17 @@ void QSvgHandler::setDefaultCoordinateSystem(LengthType type)
 {
     m_defaultCoords = type;
 }
+
+void QSvgHandler::pushColor(const QColor &color)
+{
+    m_colorStack.push(color);
+    m_colorTagCount.push(1);
+}
+
+QColor QSvgHandler::currentColor() const
+{
+    if (!m_colorStack.isEmpty())
+        return m_colorStack.top();
+    else
+        return QColor(0, 0, 0);
+}
