@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -883,8 +883,23 @@ void QDesignerActions::showHelp(const QString &url)
     if (!m_assistantClient)
         m_assistantClient
             = new QAssistantClient(QLibraryInfo::location(QLibraryInfo::BinariesPath), this);
-    m_assistantClient->showPage(QLibraryInfo::location(QLibraryInfo::DocumentationPath)
-                                + QLatin1String("/html/") + url);
+
+    QString filePath = QLibraryInfo::location(QLibraryInfo::DocumentationPath)
+                                + QLatin1String("/html/") + url;
+
+    QString cleanFilePath;
+    int index = filePath.lastIndexOf(QLatin1Char('#'));
+    if (index != -1)
+        cleanFilePath = filePath.left(index);
+    else
+        cleanFilePath = filePath;
+    
+    if (!QFile::exists(cleanFilePath)) {
+        filePath = QLibraryInfo::location(QLibraryInfo::DocumentationPath)
+                                + QLatin1String("/html/designer-manual.html");
+    }
+
+    m_assistantClient->showPage(filePath);
 }
 
 void QDesignerActions::aboutDesigner()
@@ -925,7 +940,10 @@ void QDesignerActions::showWidgetSpecificHelp()
     if (!currentPropertyName.isEmpty()) {
         QDesignerPropertySheetExtension *ps
             = qt_extension<QDesignerPropertySheetExtension *>(core()->extensionManager(),
-                                                              fw->cursor()->selectedWidget(0));
+                                                            core()->propertyEditor()->object());
+        if (!ps)
+            ps = qt_extension<QDesignerPropertySheetExtension *>(core()->extensionManager(),
+                                                            fw->cursor()->selectedWidget(0));
         Q_ASSERT(ps);
         className = ps->propertyGroup(ps->indexOf(currentPropertyName));
     } else {
@@ -934,10 +952,22 @@ void QDesignerActions::showWidgetSpecificHelp()
         className = dbi->name();
     }
 
+    // ### generalize using the Widget Data Base
+    if (className == QLatin1String("Line"))
+        className = QLatin1String("QFrame");
+    else if (className == QLatin1String("Spacer"))
+        className = QLatin1String("QSpacerItem");
+    else if (className == QLatin1String("QLayoutWidget"))
+        className = QLatin1String("QLayout");
+
     QString url = className.toLower();
+
+    // special case
     url += QLatin1String(".html");
+
     if (!currentPropertyName.isEmpty())
         url += QLatin1Char('#') + currentPropertyName;
+
     showHelp(url);
 }
 
